@@ -1,4 +1,4 @@
-from typing import List, Union, Optional, Dict, MutableMapping, Any, Set
+from typing import List, Union, Optional, Dict, MutableMapping, Any, Set, Tuple
 import itertools
 import math
 import numbers
@@ -8,6 +8,8 @@ import warnings
 
 import attr
 
+from .load_dump import dumps as demes_dumps
+
 Number = Union[int, float]
 Name = str
 Time = Number
@@ -161,8 +163,8 @@ class Epoch:
         If ``start_size != end_size``, the population size changes
         monotonically between the start and end times.
     :ivar str size_function: The size change function. This is either
-        ``constant`` or ``exponential``, though it is possible that
-        additional values will be added in the future.
+        ``constant``, ``exponential`` or ``linear``, though it is possible
+        that additional values will be added in the future.
 
          * ``constant``: the deme's size does not change over the epoch.
          * ``exponential``: the deme's size changes exponentially from
@@ -175,12 +177,15 @@ class Epoch:
                dt = (epoch.start_time - t) / epoch.time_span
                 r = math.log(epoch.end_size / epoch.start_size)
                 N = epoch.start_size * math.exp(r * dt)
+         * ``linear``: the deme's size changes linearly from
+           ``start_size`` to ``end_size`` over the epoch.
+           If :math:`t` is a time within the span of the epoch,
+           the deme size :math:`N` at :math:`t` can be calculated as:
 
-        .. warning::
+           .. code::
 
-            Do not assume an exponentially changing size just because
-            ``start_size != end_size``. For forwards compatibility,
-            applications should always check the ``size_function``.
+               dt = (epoch.start_time - t) / epoch.time_span
+                N = epoch.start_size + (epoch.end_size - epoch.start_size) * dt
 
     :ivar float selfing_rate: The selfing rate for this epoch.
     :ivar float cloning_rate: The cloning rate for this epoch.
@@ -191,7 +196,7 @@ class Epoch:
     start_size: Size = attr.ib(validator=[int_or_float, positive, finite])
     end_size: Size = attr.ib(validator=[int_or_float, positive, finite])
     size_function: str = attr.ib(
-        validator=attr.validators.in_(["constant", "exponential"])
+        validator=attr.validators.in_(["constant", "exponential", "linear"])
     )
     selfing_rate: Proportion = attr.ib(
         default=0, validator=[int_or_float, unit_interval]
@@ -1138,6 +1143,36 @@ class Deme:
         """
         return self.start_time - self.end_time
 
+    def size_at(self, time: float) -> float:
+        """
+        Get the size of the deme at a given time.
+
+        :param float time: The time at which the size should be calculated.
+        :return: The deme size.
+        :rtype: float
+        """
+        for epoch in self.epochs:
+            if epoch.start_time >= time >= epoch.end_time:
+                break
+        else:
+            raise ValueError(
+                f"time {time} is outside deme {self.name}'s existence interval: "
+                f"({self.start_time}, {self.end_time}]"
+            )
+
+        if math.isclose(time, epoch.end_time) or epoch.size_function == "constant":
+            N = epoch.end_size
+        elif epoch.size_function == "exponential":
+            dt = (epoch.start_time - time) / epoch.time_span
+            r = math.log(epoch.end_size / epoch.start_size)
+            N = epoch.start_size * math.exp(r * dt)
+        elif epoch.size_function == "linear":
+            dt = (epoch.start_time - time) / epoch.time_span
+            N = epoch.start_size + (epoch.end_size - epoch.start_size) * dt
+        else:
+            raise NotImplementedError(f"unknown size_function '{epoch.size_function}'")
+        return N
+
 
 @attr.s(auto_attribs=True, kw_only=True, slots=True)
 class Graph:
@@ -1191,7 +1226,12 @@ class Graph:
     migrations: List[AsymmetricMigration] = attr.ib(factory=list, init=False)
     pulses: List[Pulse] = attr.ib(factory=list, init=False)
 
-    _deme_map: Dict[Name, Deme] = attr.ib(factory=dict, init=False)
+    # This attribute is for internal use only. It's a (hidden) attribute
+    # because we're using slotted classes and can't add attributes after
+    # object creation (e.g. in __attrs_post_init__()).
+    _deme_map: Dict[Name, Deme] = attr.ib(
+        factory=dict, init=False, repr=False, cmp=False
+    )
 
     def __attrs_post_init__(self):
         if self.time_units != "generations" and self.generation_time is None:
@@ -1199,18 +1239,41 @@ class Graph:
                 'if time_units!="generations", generation_time must be specified'
             )
 
-    def __getitem__(self, deme_name):
+    def __getitem__(self, deme_name: Name) -> Deme:
         """
-        Return the :class:`.Deme` with the specified name.
+        Get the :class:`.Deme` with the specified name.
+
+        .. code::
+
+            graph = demes.load("gutenkunst_ooa.yml")
+            yri = graph["YRI"]
+            print(yri)
+
+        :param str deme_name: The name of the deme.
+        :rtype: Deme
+        :return: The deme.
         """
         return self._deme_map[deme_name]
 
-    def __contains__(self, deme_name):
+    def __contains__(self, deme_name: Name) -> bool:
         """
         Check if the graph contains a deme with the specified name.
+
+        .. code::
+
+            graph = demes.load("gutenkunst_ooa.yml")
+            if "CHB" in graph:
+                print("Deme CHB is in the graph")
+
+        :param str deme_name: The name of the deme.
+        :rtype: bool
+        :return: ``True`` if the deme is in the graph, ``False`` otherwise.
         """
         return deme_name in self._deme_map
 
+    # Use the simplified YAML output as the string representation.
+    __str__ = demes_dumps
+
     def assert_close(
         self,
         other,
@@ -1568,16 +1631,62 @@ class Graph:
         self.pulses.append(new_pulse)
         return new_pulse
 
-    def _migration_matrices(self):
+    def migration_matrices(self) -> Tuple[List[List[List[float]]], List[Number]]:
         """
-        Return a list of migration matrices, and a list of end times that
-        partition them. The start time for the first matrix is inf.
+        Get the migration matrices and the end times that partition them.
+
+        Returns a list of matrices, one for each time interval
+        over which migration rates do not change, in time-descending
+        order (from most ancient to most recent). For a migration matrix list
+        :math:`M`, the migration rate is :math:`M[i][j][k]` from deme
+        :math:`k` into deme :math:`j` during the :math:`i` 'th time interval.
+        The order of the demes' indices in each matrix matches the
+        order of demes in the graph's deme list (I.e. deme :math:`j`
+        corresponds to ``Graph.demes[j]``).
+
+        There is always at least one migration matrix in the list, even when
+        the graph defines no migrations.
+
+        A list of end times to which the matrices apply is also
+        returned. The time intervals to which the migration rates apply are an
+        open-closed interval ``(start_time, end_time]``, where the start time
+        of the first matrix is ``inf`` and the start time of subsequent
+        matrices match the end time of the previous matrix in the list.
+
+        .. note::
+            The last entry of the list of end times is always ``0``,
+            even when all demes in the graph go extinct before time ``0``.
+
+
+        .. code::
+
+            graph = demes.load("gutenkunst_ooa.yml")
+            mm_list, end_times = graph.migration_matrices()
+            start_times = [math.inf] + end_times[:-1]
+            assert len(mm_list) == len(end_times) == len(start_times)
+            deme_ids = {deme.name: j for j, deme in enumerate(graph.demes)}
+            j = deme_ids["YRI"]
+            k = deme_ids["CEU"]
+            for mm, start_time, end_time in zip(mm_list, start_times, end_times):
+                print(
+                    f"CEU -> YRI migration rate is {mm[j][k]} during the "
+                    f"time interval ({start_time}, {end_time}]"
+                )
+
+        :return: A 2-tuple of ``(mm_list, end_times)``,
+            where ``mm_list`` is a list of migration matrices,
+            and ``end_times`` are a list of end times for each matrix.
+        :rtype: tuple[list[list[list[float]]], list[float]]
         """
         uniq_times = set(migration.start_time for migration in self.migrations)
         uniq_times.update(migration.end_time for migration in self.migrations)
-        end_times = sorted(uniq_times, reverse=True)[1:]
+        uniq_times.discard(math.inf)
+        end_times = sorted(uniq_times, reverse=True)
+        if len(end_times) == 0 or end_times[-1] != 0:
+            # Extend to t=0 even when there are no migrations.
+            end_times.append(0)
         n = len(self.demes)
-        mm_list = [[[0] * n for _ in range(n)] for _ in range(len(end_times))]
+        mm_list = [[[0.0] * n for _ in range(n)] for _ in range(len(end_times))]
         deme_id = {deme.name: j for j, deme in enumerate(self.demes)}
         for migration in self.migrations:
             start_time = math.inf
@@ -1593,7 +1702,7 @@ class Graph:
                             f"source={migration.source}, dest={migration.dest} "
                             f"between start_time={start_time}, end_time={end_time}"
                         )
-                    mm_list[k][dest_id][source_id] = migration.rate
+                    mm_list[k][dest_id][source_id] = float(migration.rate)
                 start_time = end_time
         return mm_list, end_times
 
@@ -1603,7 +1712,7 @@ class Graph:
         deme in any interval of time.
         """
         start_time = math.inf
-        mm_list, end_times = self._migration_matrices()
+        mm_list, end_times = self.migration_matrices()
         for migration_matrix, end_time in zip(mm_list, end_times):
             for j, row in enumerate(migration_matrix):
                 row_sum = sum(row)
